#version 330

in vec2 fragmentUV;
in vec3 P;
in vec3 N;
in vec3 C;

uniform vec3 cameraPosition;

const int lightCount = 1;
uniform vec3 position_light[lightCount];

uniform sampler2D colorTexture;

uniform vec3 objectColor;

out vec4 fragColor;

void main() {

	vec3 textureColor = texture(colorTexture, fragmentUV).rgb;

	vec3 normalDirection = normalize(N);

	vec3 viewDirection = normalize(cameraPosition - P);

	vec3 finalToonColor = vec3(0.0f,0.0f,0.0f);
	
	for (int index = 0; index < lightCount; index++) {

		vec3 vertexToLightSource = vec3(position_light[index] - P);
		float distance = length(vertexToLightSource);
		float attenuation = 1.0f / distance;
		vec3 lightDirection = normalize(vertexToLightSource);
		
		float intensity =   max(0.0f, dot(normalDirection, lightDirection));

		float toonFactor = 1.0f;

		if(intensity > 0.95f) {
			toonFactor = 0.75f;
		} else if(intensity > 0.5f) {
			toonFactor = 0.5f;
		} else if(intensity > 0.25f) {
			toonFactor = 0.25f;
		} else {
			toonFactor = 0.11f;
		}

		finalToonColor = finalToonColor + (toonFactor * vec3(textureColor));

	}


	fragColor = 2.0 * vec4(finalToonColor, 1.0f) * vec4(objectColor, 1.0f) ;


}


