#version 330

in vec2 fragmentUV;
in vec3 P;
in vec3 N;
in mat3 localSurface2World;
in vec3 tang_test;
in mat3 normalMatrix;
in vec4 ShadowCoord;

uniform vec3 cameraPosition;

const int lightCount = 1;
uniform vec3 position_light[lightCount];

uniform vec3 _Color = vec3(1.0, 1.0, 1.0);
uniform vec4 _SpecColor = vec4(1,1,1,1);
uniform float _Shininess = 10;
uniform vec3 _LightColor0 = vec3(1.0,1.0,1.0);

uniform sampler2D colorTexture;
uniform sampler2D normalMap;
uniform sampler2D shadowMap;

out vec4 fragColor;

void main() {
	
	// load texcolor
	vec3 diffuseColor = texture(colorTexture, fragmentUV).rgb;

	// load normalmap
	vec3 normalMapVector = texture(normalMap, fragmentUV).rgb;

	// local normalcoords
	vec3 normalMapVectorNew = vec3((2 * normalMapVector.x) - 1, (2 * normalMapVector.y) - 1, (2 * normalMapVector.z) - 1);

	// local normal transform
    vec3 normal_dir =  normalize(localSurface2World * normalMapVectorNew);


	// blinn phong
	vec3 view_dir = normalize(cameraPosition - P);

	vec3 vertexToLightSource = vec3(position_light[0] - P);
	float distance = length(vertexToLightSource);
	float attenuation = 1.0 / distance;
	vec3 lightDirection = normalize(vertexToLightSource);

	
	// vec3 ambientLighting = vec3(0.2f,0.2f,0.2f) * vec3(_Color);
	vec3 ambientLighting = vec3(0.5f,0.5f,0.5f) * vec3(_Color);

	vec3 diffuseReflection = 
               attenuation * vec3(_LightColor0) * vec3(_Color) 
               * max(0.0, dot(normal_dir, lightDirection));

	 vec3 specularReflection;
            if (dot(normal_dir, lightDirection) < 0.0) 
               // light source on the wrong side?
            {
               specularReflection = vec3(0.0, 0.0, 0.0); 
                  // no specular reflection
            }
            else // light source on the right side
            {
               specularReflection = attenuation * vec3(_LightColor0) 
                  * vec3(_SpecColor) * pow(max(0.0, dot(
                  reflect(-lightDirection, normal_dir), 
                  view_dir)), _Shininess);
            }

	float visibility = 1.0;
    if ( texture( shadowMap, vec2(ShadowCoord.x, ShadowCoord.y) ).z  <  ShadowCoord.z){
    visibility = 0.1;
    }
	//float visibility = texture( shadowMap, vec3(ShadowCoord.xy, (ShadowCoord.z)/ShadowCoord.w) );
	// fragColor =  vec4(ambientLighting  + diffuseReflection * 3 + specularReflection, 1.0) * vec4(diffuseColor, 1.0) * visibility;
	fragColor =  vec4(ambientLighting  + diffuseReflection * 8 + specularReflection, 1.0) * vec4(diffuseColor, 1.0) * visibility;
	// fragColor =  vec4(ambientLighting  + diffuseReflection + specularReflection, 1.0) * vec4(diffuseColor, 1.0);

	// fragColor = vec4(localSurface2World,1.0);
	//fragColor = vec4(normalMapVectorNew,1.0); // lokale Normalen der Normal Map passen
	// fragColor = vec4(normal_dir,1.0); // Bei Multiplikation mit TBN Matrix erhalte ich Nullwerte?
	//vec3 test = vec3((normalize(normal_dir) + 1.0) / 2.0);
	// vec3 test = vec3((normal_dir + 1.0) / 2.0);

	// fragColor = vec4(test,1.0);


}